////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////

#include <gtf/pluginloader/PluginSymbols.h>
#include <gtf/dependencyresolver/DependencyContainer.h>
#include <gtf/osal/MainTaskDispatcher.h>
#include <gtf/datapool/Datapool.h>
#include <gtf/eventsystem/EventQueue.h>
#include "HmiAppInteractionDemo.h"

GTF_PLUGIN_SO_SYMBOL const char * GTF_PLUGIN_GET_VERSION()
{
    return GTF_PRODUCT_STRING;
}

static gtf::dependencyresolver::InterfaceHandle CreateApplicationInteractionDemo(gtf::dependencyresolver::DependencyContainerHandle & container)
{
    if (!container.valid())
    {
        return gtf::dependencyresolver::InterfaceHandle();
    }

    gtf::osal::MainTaskDispatcherHandle mainDispatcher = container->get<gtf::osal::MainTaskDispatcher>();
    if (!mainDispatcher.valid())
    {
        return gtf::dependencyresolver::InterfaceHandle();
    }

    gtf::datapool::DatapoolHandle datapool = container->get<gtf::datapool::Datapool>();
    if (!datapool.valid())
    {
        return gtf::dependencyresolver::InterfaceHandle();
    }

    gtf::eventsystem::global::EventQueueHandle globalEventQueue = container->get<gtf::eventsystem::global::EventQueue>();
    if (!globalEventQueue.valid())
    {
        return gtf::dependencyresolver::InterfaceHandle();
    }

    return gtf::dependencyresolver::InterfaceHandle(new applicationInteractionDemo::ApplicationInteractionDemo(container, mainDispatcher, datapool, globalEventQueue));
}

GTF_PLUGIN_SO_SYMBOL void GTF_PLUGIN_EXTEND_CATALOG(void * catalog)
{
    if (!catalog)
    {
        return;
    }

    gtf::dependencyresolver::DependencyCatalog & dependencyCatalog = *static_cast<gtf::dependencyresolver::DependencyCatalog*>(catalog);

    gtf::dependencyresolver::extendCatalog<gtf::launcher::Runnable>(dependencyCatalog, CreateApplicationInteractionDemo);
}

