////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////
#ifndef EXAMPLE_APP_H_INCLUDED
#define EXAMPLE_APP_H_INCLUDED

#include <gtf/launcher/Runnable.h>
#include <gtf/osal/MainTaskDispatcher.h>
#include <gtf/osal/DelegateTask.h>
#include <gtf/eventsystem/EventQueue.h>
#include <gtf/datapool/Datapool.h>
#include <gtf/resourcer/Resourcer.h>
#include <gtf/configuration/PathValue.h>

#include "MandelbrotSet.h"
#include "Mandelbrot2MemImageConverter.h"

namespace  applicationInteractionDemo
{

    class ApplicationInteractionDemo : public gtf::launcher::Runnable
    {
    public:
        ApplicationInteractionDemo(gtf::dependencyresolver::DependencyContainerHandle & container_
            , gtf::osal::MainTaskDispatcherHandle & taskDispatcher_
            , gtf::datapool::DatapoolHandle & datapool_
            , gtf::eventsystem::global::EventQueueHandle & eventQueue_);
        virtual void run();
        virtual void stop();

    private:
        void processItems();
        void processLoop();
        void invoker();
        void handleEvents(gtf::eventsystem::global::EventHandle event);
        void initializeDatapool();

        void readAllDatapoolItems();
        void loadMandelBrot();
        void updateWindowList();
        void changeBooleanValues();
        void changeIntValues();
        void changeStringValues();
        void loadExternalImage();
        void loadDynamicImage();

        gtf::configuration::PathValueHandle getImagePath();
        gtf::configuration::PathValueHandle getDatapoolDescription();

        gtf::dependencyresolver::DependencyContainerHandle container;
        gtf::osal::MainTaskDispatcherHandle taskDispatcher;
        gtf::datapool::DatapoolHandle datapool;
        gtf::eventsystem::global::EventQueueHandle eventQueue;
        gtf::osal::DelegateTask processTask;
        gtf::osal::DelegateTask processLoopTask;
        gtf::resourcer::ResourcerHandle resourcer;
        gtf::configuration::PathValueHandle imagePath;
        MandelbrotSet imageCreator;
        gtf::resourcer::ResourceFormat memImageType;
        gtf::resourcer::ResourceFormat mandelbrotImageType;
        Mandelbrot2MemImageConverter imageConverter;
        uint32_t loopCounter;
    };
}

#endif // EXAMPLE_APP_H_INCLUDED

