////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////

#ifndef MANDELBROT_2_MEM_IMAGE_CONVERTER_H_INCLUDED
#define MANDELBROT_2_MEM_IMAGE_CONVERTER_H_INCLUDED

#include <gtf/resourcer/ResourceLoader.h>
#include <gtf/resources/ImageResource.h>

/*!
 * \brief Converts MandelbrotImage to GtfMemImage
 *
 * This GtfResourceLoader converts MandelbrotImage objects into gtf_men_image.
 */
class Mandelbrot2MemImageConverter : public gtf::resourcer::ResourceLoader
{
public:
    Mandelbrot2MemImageConverter();

    void setDestinationFormat(gtf::resourcer::ResourceFormat format);

    /*!
     * \name GtfResourceLoader Interface
     * \{
     */
    /*!
     * \brief Converts a MandelbrotImage into a gtf_men_image.
     *
     * Expects \a pBase to be a pointer to a MandelbrotImage, returns a pointer to a
     * GtfMemImage
     */
    virtual gtf::smartptr::RefCountedPtr<gtf::resourcer::ResourceData> load(gtf::smartptr::RefCountedPtr<gtf::resourcer::ResourceData> const& base, resourcerError &errorOut);
    /*!
     * \brief Unloads GtfMemImage objects created by this class
     *
     * Expects a pointer to a GtfMemImage, and frees it.
     * Will be called with the object returned from Mandelbrot2MemImageConverter::load
     */
    virtual void unload(gtf::resourcer::ResourceData const* pDest);
    /*!
     * \}
     */
private:

    class HeapImageData : public gtf::resources::MemImage::ImageDataProxy
    {
    public:
        HeapImageData(uint32_t size)
            : ImageDataProxy(new uint8_t[size])
        {}

        ~HeapImageData()
        {
            delete [] image;
        }

    private:
        HeapImageData &operator=(HeapImageData const &);
        HeapImageData(HeapImageData const &);
    };

    static const uint32_t RainbowGradientSize = 1024;
    uint32_t rainbow[RainbowGradientSize];
    gtf::resourcer::ResourceFormat m_destinationFormat;
};

#endif // MANDELBROT_2_MEM_IMAGE_CONVERTER_H_INCLUDED

