////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////

#include <stdlib.h>
#include <complex>
#include "MandelbrotImage.h"


MandelbrotImage::MandelbrotImage(gtf::resourcer::ResourceFormat format
                                , gtf::resourcer::ResourceLoader *unloader
                                , uint32_t width_
                                , uint32_t height_
                                , value_type posX
                                , value_type posY
                                , value_type scale
                                , uint32_t maxIterations)
    : gtf::resourcer::ResourceData(format, unloader)
    , width(width_)
    , height(height_)
    , pixelCount(width*height)
{
    if (pixelCount == 0)
    {
        pixelData = NULL;
        return;
    }
    pixelData = (value_type*)malloc(pixelCount*sizeof(value_type));
    if (pixelData == NULL)
    {
        return;
    }
    draw_Mandelbrot( posX - value_type(width_)/scale, posX + value_type(width_)/scale
                   , posY - value_type(height_)/scale, posY + value_type(height_)/scale
                   , maxIterations
                   );
}

MandelbrotImage::~MandelbrotImage()
{
    free(pixelData);
}

void MandelbrotImage::draw_Mandelbrot(value_type cxmin
                                     , value_type cxmax
                                     , value_type cymin
                                     , value_type cymax
                                     , uint32_t maxIterations)
{
    uint32_t const ixsize = getWidth();
    uint32_t const iysize = getHeight();
    value_type scaleX = 1.0/(value_type(ixsize)-1.0)*(cxmax-cxmin);
    value_type scaleY = 1.0/(value_type(iysize)-1.0)*(cymax-cymin);
    value_type escape_radius = 20.0f;
    value_type log_2 = log(2.0f);
    value_type* pImage = pixelData;

    for (uint32_t iy = 0; iy < iysize; ++iy)
    {
        value_type scaledImaginary = cymin + value_type(iy)*scaleY;
        for (uint32_t ix = 0; ix < ixsize; ++ix)
        {
            std::complex<value_type> c(cxmin + value_type(ix)*scaleX,scaledImaginary  );
            std::complex<value_type> z = 0;
            uint32_t iterations;

            for (iterations = 0
                ; iterations < maxIterations && abs(z) < escape_radius
                ; ++iterations)
            {
                z = z*z + c;
            }

            *pImage++ = value_type(iterations) - (log (log(abs(z))/ log_2));
        }
    }
}
