////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////

#include <assert.h>
#include <cmath>
#include "MandelbrotSet.h"
#include "MandelbrotImage.h"


MandelbrotSet::MandelbrotSet()
{
}

gtf::smartptr::RefCountedPtr<gtf::resourcer::ResourceData> MandelbrotSet::load(gtf::smartptr::RefCountedPtr<gtf::resourcer::ResourceData> const& base, resourcerError &errorOut)
{
    GTF_UNUSED_PARAM(base)
    GTF_UNUSED_PARAM(errorOut)

    return NULL;
}

void MandelbrotSet::unload(gtf::resourcer::ResourceData const* destination)
{
    delete static_cast<MandelbrotImage const*>(destination);
    // this method is called when the resource handle used to store the
    // MandelbrotImage pImage is no longer referenced by anyone.
    // this cast is not necessary - it shall higlight that this method
    // will only be called with instances for ResourceData objects
    // created within this class.
}

gtf::smartptr::RefCountedPtr<MandelbrotImage> MandelbrotSet::createMandelbrotImage(gtf::resourcer::ResourceFormat format, uint32_t timeStep)
{
    const uint32_t stepOfReturn = 40;

    timeStep = timeStep % (2 * stepOfReturn);

    if(timeStep > stepOfReturn)
    {
        timeStep = stepOfReturn - (timeStep % stepOfReturn);
    }

    double scale = 2.0 + pow(1.2,double(timeStep)/2.0);

    MandelbrotSet::value_type middleArcOfMandelbrotSet_X = -0.15550494345;
    MandelbrotSet::value_type middleArcOfMandelbrotSet_Y = 0.65017043818;
    return new MandelbrotImage(format, this, 128, 128, middleArcOfMandelbrotSet_X, middleArcOfMandelbrotSet_Y, scale, stepOfReturn);
}
