////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////

#ifndef MANDELBROT_SET_H_INCLUDED
#define MANDELBROT_SET_H_INCLUDED

#include <gtf/types/TypesC99.h>
#include <gtf/utils/smartptr/RefCountedPtr.h>
#include <gtf/resourcer/ResourceFormat.h>
#include <gtf/resourcer/ResourceLoader.h>

class MandelbrotImage;

/*!
 * \brief Provides Mandelbrot images, and frees Mandelbrot images.
 *
 * Deallocation of Mandelbrot images happes through the GtfResourceLoader interface.
 */
class MandelbrotSet : public gtf::resourcer::ResourceLoader
{
public:
    typedef double value_type;
    MandelbrotSet();

    /*!
     * \name GtfResourceLoader interface:
     * \{
     */
    /*!
     * \brief Never called by the system.
     */
    virtual gtf::smartptr::RefCountedPtr<gtf::resourcer::ResourceData> load(gtf::smartptr::RefCountedPtr<gtf::resourcer::ResourceData> const& base, resourcerError &errorOut);
    /*!
     * \brief Unloads the MandelbrotImage objects created by this class.
     *
     * Expects \a destination to be a pointer to a MandelbrotImage
     */
    virtual void unload(gtf::resourcer::ResourceData const* destination);
    /*!
     * \}
     */

    /*!
     * \brief Create a MandelbrotImage for the given \a timestep
     *
     * MandelbrotSet creates a zoom animation that zooms into a certain
     * point of the mandelbrot set.
     */
    gtf::smartptr::RefCountedPtr<MandelbrotImage> createMandelbrotImage(gtf::resourcer::ResourceFormat format, uint32_t timeStep);
};

#endif // MANDELBROT_SET_H_INCLUDED
