////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////
#include <gtf/pluginloader/PluginSymbols.h>
#include <gtf/pluginloader/PluginData.h>

#include <gtf/launcher/Runnable.h>

#include "TemperatureConverterImpl.h"
#include "TemperatureConverterApp.h"

static gtf::dependencyresolver::InterfaceHandle registerConverter(gtf::dependencyresolver::DependencyContainerHandle & container)
{
    if (!container.valid())
    {
        return gtf::dependencyresolver::InterfaceHandle();
    }
    gtf::diagnostic::LogHandle logger = container->get<gtf::diagnostic::Log>();

    if (!logger.valid())
    {
        return gtf::dependencyresolver::InterfaceHandle();
    }

    dependencyResolving::TemperatureConverterHandle temperatureConverter(new dependencyResolving::TemperatureConverterImpl(logger));

    // add the TemperatureConverter instance to the dependency resolver container
    container->registerInstance<dependencyResolving::TemperatureConverter>(temperatureConverter);
    return temperatureConverter;
}

static gtf::dependencyresolver::InterfaceHandle CreateTemperatureConverterApplication(gtf::dependencyresolver::DependencyContainerHandle & container)
{
    if (!container.valid())
    {
        return gtf::dependencyresolver::InterfaceHandle();
    }

    dependencyResolving::TemperatureConverterHandle converter = container->get<dependencyResolving::TemperatureConverter>();

    if (!converter.valid())
    {
        return gtf::dependencyresolver::InterfaceHandle();
    }

    return gtf::dependencyresolver::InterfaceHandle(new dependencyResolving::TemperatureConverterApplication(converter));
}

GTF_PLUGIN_SO_SYMBOL const char * GTF_PLUGIN_GET_VERSION()
{
    return GTF_PRODUCT_STRING;
}

GTF_PLUGIN_SO_SYMBOL void GTF_PLUGIN_EXTEND_CATALOG(void * catalog)
{
    if (!catalog)
    {
        return;
    }

    gtf::dependencyresolver::DependencyCatalog & dependencyCatalog = *static_cast<gtf::dependencyresolver::DependencyCatalog*>(catalog);
    gtf::dependencyresolver::extendCatalog<dependencyResolving::TemperatureConverter>(dependencyCatalog, registerConverter);
    gtf::dependencyresolver::extendCatalog<gtf::launcher::Runnable>(dependencyCatalog, CreateTemperatureConverterApplication);
}
