////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////
#include "EventApp.h"
#include <gtf/eventsystem/ScalarValue.h>

#include "global_events.h"

eventplugin::EventApplication::EventApplication(gtf::osal::MainTaskDispatcherHandle & taskDispatcher_, gtf::eventsystem::global::EventQueueHandle & eventQueue_)
: taskDispatcher(taskDispatcher_)
, eventQueue(eventQueue_)
{
    // setup the event processing task
    processEventTask.setExecuteCallback(gtf::utils::Delegate<void()>::create<gtf::eventsystem::global::EventQueue, &gtf::eventsystem::global::EventQueue::dispatch>(&*eventQueue));
}

void eventplugin::EventApplication::run()
{
    //unsubscrie for the event and specify the callback method
    eventQueue->subscribe(gtf::eventsystem::SubscriptionType::Event
        , GLOBAL_EVENT_GRP_ID_whatToDo
        , GLOBAL_EVENT_MSG_ID_whatToDo
        , this
        , gtf::utils::Delegate<void(gtf::eventsystem::global::EventHandle)>::create<EventApplication, &EventApplication::processEvent>(this));

    //
    eventQueue->setInvoker(gtf::utils::Delegate<void()>::create<EventApplication, &EventApplication::invoker>(this));
}

void eventplugin::EventApplication::stop()
{
    eventQueue->unsubscribe(gtf::eventsystem::SubscriptionType::Event, GLOBAL_EVENT_GRP_ID_whatToDo, GLOBAL_EVENT_MSG_ID_whatToDo, this);
}

void eventplugin::EventApplication::invoker()
{
    //start the event processing task
    taskDispatcher->performTask(processEventTask);
}

void eventplugin::EventApplication::processEvent(gtf::eventsystem::global::EventHandle event)
{
    static int32_t count = 0;
    if (count % 2 == 1)
    {
        gtf::eventsystem::global::EventHandle returnEvent = eventQueue->createEvent(GLOBAL_EVENT_GRP_ID_goToA, GLOBAL_EVENT_MSG_ID_goToA);
        if (returnEvent.valid())
        {
            eventQueue->sendEvent(returnEvent);
        }
    }
    else
    {
        gtf::eventsystem::global::EventHandle returnEvent = eventQueue->createEvent(GLOBAL_EVENT_GRP_ID_goToB, GLOBAL_EVENT_MSG_ID_goToB);
        if (returnEvent.valid())
        {
            eventQueue->sendEvent(returnEvent);
        }
    }
    count += 1;
}


