////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////

#include "LoadedStageImpl.h"
#include <gtf/type/TypeId.h>
#include <gtf/type/ParameterStack.h>
#include <gtf/metainfo/ActionDescriptor.h>
#include <gtf/coreruntime/CoreModel.h>
#include <gtf/coreruntime/CoreRuntime.h>

static void example_sum(void*, gtf::type::ParameterStack& stack)
{
    int32_t operand1, operand2, result = 0;

    stack.Scalar_PopValue(gtf::type::TypeId::eTypeId_int32, &operand2, sizeof(operand2));
    stack.Scalar_PopValue(gtf::type::TypeId::eTypeId_int32, &operand1, sizeof(operand1));

    result = operand1 + operand2;

    stack.Scalar_PushValue(gtf::type::TypeId::eTypeId_int32, &result, sizeof(result));
}

scriptfunction::LoadedStageImpl::LoadedStageImpl(gtf::dependencyresolver::DependencyContainerHandle& container_)
:container(container_)
{
    container->getAll<gtf::coreruntime::CoreRuntime>(runtimes);

    gtf::coreruntime::CoreModelHandle model = container->get<gtf::coreruntime::CoreModel>();

    if (!model.valid())
    {
        return;
    }

    uint32_t actionId = model->getActionId("ExampleFunction_SUM");

    if (actionId != gtf::types::limits::uint32_max)
    {
        for (gtf::container::Array<gtf::coreruntime::CoreRuntimeHandle>::Iterator it = runtimes.begin(), e = runtimes.end(); it != e; ++it)
        {
            (*it)->setActionImplementation
                (actionId
                , gtf::metainfo::ActionDescriptor::Implementation(NULL, &example_sum)
                );
        }
    }
}

scriptfunction::LoadedStageImpl::~LoadedStageImpl()
{

}

