////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////

#include <gtf/pluginloader/PluginSymbols.h>
#include <gtf/dependencyresolver/DependencyContainer.h>
#include "PreparedStageImpl.h"
#include "LoadedStageImpl.h"

GTF_PLUGIN_SO_SYMBOL const char * GTF_PLUGIN_GET_VERSION()
{
    return GTF_PRODUCT_STRING;
}

static gtf::dependencyresolver::InterfaceHandle createPreparedStageImpl(gtf::dependencyresolver::DependencyContainerHandle & container)
{
    if (!container.valid())
    {
        return gtf::dependencyresolver::InterfaceHandle();
    }
    return gtf::dependencyresolver::InterfaceHandle(new scriptfunction::PreparedStageImpl(container));
}

static gtf::dependencyresolver::InterfaceHandle createLoadedStageImpl(gtf::dependencyresolver::DependencyContainerHandle & container)
{
    if (!container.valid())
    {
        return gtf::dependencyresolver::InterfaceHandle();
    }
    return gtf::dependencyresolver::InterfaceHandle(new scriptfunction::LoadedStageImpl(container));
}

GTF_PLUGIN_SO_SYMBOL void GTF_PLUGIN_EXTEND_CATALOG(void * catalog)
{
    if (!catalog)
    {
        return;
    }

    gtf::dependencyresolver::DependencyCatalog & dependencyCatalog = *static_cast<gtf::dependencyresolver::DependencyCatalog*>(catalog);
    gtf::dependencyresolver::extendCatalog<gtf::model::PreparedStage>(dependencyCatalog, createPreparedStageImpl);
    gtf::dependencyresolver::extendCatalog<gtf::model::LoadedStage>(dependencyCatalog, createLoadedStageImpl);
}


