////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////

#include <gtf/pluginloader/PluginSymbols.h>
#include <gtf/dependencyresolver/DependencyContainer.h>
#include "EarlyStage.h"
#include "EarlyStageProvider.h"
#include "LateStage.h"
#include "LateStageProvider.h"
#include "PreparedStageImpl.h"
#include <gtf/model/PreparedStage.h>

GTF_PLUGIN_SO_SYMBOL const char * GTF_PLUGIN_GET_VERSION()
{
    return GTF_PRODUCT_STRING;
}

static gtf::dependencyresolver::InterfaceHandle createEarlyStageProvider(gtf::dependencyresolver::DependencyContainerHandle & )
{
    return gtf::dependencyresolver::InterfaceHandle(new stageplugin::EarlyStageStageProvider());
}

static gtf::dependencyresolver::InterfaceHandle createEarlyStage(gtf::dependencyresolver::DependencyContainerHandle & container)
{
    if (!container.valid())
    {
        return gtf::dependencyresolver::InterfaceHandle();
    }

    return gtf::dependencyresolver::InterfaceHandle(new stageplugin::EarlyStage(container));
}

static gtf::dependencyresolver::InterfaceHandle createLateStageProvider(gtf::dependencyresolver::DependencyContainerHandle &)
{
    return gtf::dependencyresolver::InterfaceHandle(new stageplugin::LateStageProvider());
}

static gtf::dependencyresolver::InterfaceHandle createLateStage(gtf::dependencyresolver::DependencyContainerHandle & container)
{
    if (!container.valid())
    {
        return gtf::dependencyresolver::InterfaceHandle();
    }

    return gtf::dependencyresolver::InterfaceHandle(new stageplugin::LateStage(container));
}

static gtf::dependencyresolver::InterfaceHandle createPrepairedStage(gtf::dependencyresolver::DependencyContainerHandle & container)
{
    if (!container.valid())
    {
        return gtf::dependencyresolver::InterfaceHandle();
    }

    return gtf::dependencyresolver::InterfaceHandle(new stageplugin::PreparedStageImpl(container));
}

GTF_PLUGIN_SO_SYMBOL void GTF_PLUGIN_EXTEND_CATALOG(void * catalog)
{
    if (!catalog)
    {
        return;
    }

    gtf::dependencyresolver::DependencyCatalog & dependencyCatalog = *static_cast<gtf::dependencyresolver::DependencyCatalog*>(catalog);
    gtf::dependencyresolver::extendCatalog<gtf::model::StageProvider>(dependencyCatalog, createEarlyStageProvider);
    gtf::dependencyresolver::extendCatalog<gtf::model::StageProvider>(dependencyCatalog, createLateStageProvider);
    gtf::dependencyresolver::extendCatalog<gtf::model::PreparedStage>(dependencyCatalog, createPrepairedStage);
    gtf::dependencyresolver::extendCatalog<stageplugin::EarlyStage>(dependencyCatalog, createEarlyStage);
    gtf::dependencyresolver::extendCatalog<stageplugin::LateStage>(dependencyCatalog, createLateStage);
}

